# coding: utf-8

"""
    Marketing Emails

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from hubspot.marketing.emails.api_client import ApiClient
from hubspot.marketing.emails.exceptions import ApiTypeError, ApiValueError  # noqa: F401


class MarketingEmailsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def archive(self, email_id, **kwargs):  # noqa: E501
        """Delete a marketing email.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.archive(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of the marketing email to delete. (required)
        :type email_id: str
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.archive_with_http_info(email_id, **kwargs)  # noqa: E501

    def archive_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Delete a marketing email.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.archive_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of the marketing email to delete. (required)
        :type email_id: str
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        local_var_params = locals()

        all_params = ["email_id", "archived"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method archive" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `archive`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []
        if local_var_params.get("archived") is not None:  # noqa: E501
            query_params.append(("archived", local_var_params["archived"]))  # noqa: E501

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {}

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def clone(self, content_clone_request_v_next, **kwargs):  # noqa: E501
        """Clone a marketing email.  # noqa: E501

        This will create a duplicate email with the same properties as the original, with the exception of a unique ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.clone(content_clone_request_v_next, async_req=True)
        >>> result = thread.get()

        :param content_clone_request_v_next: (required)
        :type content_clone_request_v_next: ContentCloneRequestVNext
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.clone_with_http_info(content_clone_request_v_next, **kwargs)  # noqa: E501

    def clone_with_http_info(self, content_clone_request_v_next, **kwargs):  # noqa: E501
        """Clone a marketing email.  # noqa: E501

        This will create a duplicate email with the same properties as the original, with the exception of a unique ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.clone_with_http_info(content_clone_request_v_next, async_req=True)
        >>> result = thread.get()

        :param content_clone_request_v_next: (required)
        :type content_clone_request_v_next: ContentCloneRequestVNext
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["content_clone_request_v_next"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method clone" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'content_clone_request_v_next' is set
        if self.api_client.client_side_validation and local_var_params.get("content_clone_request_v_next") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `content_clone_request_v_next` when calling `clone`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        if "content_clone_request_v_next" in local_var_params:
            body_params = local_var_params["content_clone_request_v_next"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # HTTP header `Content-Type`
        content_types_list = local_var_params.get("_content_type", self.api_client.select_header_content_type(["application/json"], "POST", body_params))  # noqa: E501
        if content_types_list:
            header_params["Content-Type"] = content_types_list

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/clone",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def create(self, email_create_request, **kwargs):  # noqa: E501
        """Create a new marketing email.  # noqa: E501

        Use this endpoint to create a new marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create(email_create_request, async_req=True)
        >>> result = thread.get()

        :param email_create_request: (required)
        :type email_create_request: EmailCreateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.create_with_http_info(email_create_request, **kwargs)  # noqa: E501

    def create_with_http_info(self, email_create_request, **kwargs):  # noqa: E501
        """Create a new marketing email.  # noqa: E501

        Use this endpoint to create a new marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_with_http_info(email_create_request, async_req=True)
        >>> result = thread.get()

        :param email_create_request: (required)
        :type email_create_request: EmailCreateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_create_request"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method create" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_create_request' is set
        if self.api_client.client_side_validation and local_var_params.get("email_create_request") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_create_request` when calling `create`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        if "email_create_request" in local_var_params:
            body_params = local_var_params["email_create_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # HTTP header `Content-Type`
        content_types_list = local_var_params.get("_content_type", self.api_client.select_header_content_type(["application/json"], "POST", body_params))  # noqa: E501
        if content_types_list:
            header_params["Content-Type"] = content_types_list

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            201: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def create_ab_test_variation(self, ab_test_create_request_v_next, **kwargs):  # noqa: E501
        """Create an A/B test variation of a marketing email.  # noqa: E501

        Create a variation of a marketing email for an A/B test. The new variation will be created as a draft. If an active variation already exists, a new one won't be created.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_ab_test_variation(ab_test_create_request_v_next, async_req=True)
        >>> result = thread.get()

        :param ab_test_create_request_v_next: (required)
        :type ab_test_create_request_v_next: AbTestCreateRequestVNext
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.create_ab_test_variation_with_http_info(ab_test_create_request_v_next, **kwargs)  # noqa: E501

    def create_ab_test_variation_with_http_info(self, ab_test_create_request_v_next, **kwargs):  # noqa: E501
        """Create an A/B test variation of a marketing email.  # noqa: E501

        Create a variation of a marketing email for an A/B test. The new variation will be created as a draft. If an active variation already exists, a new one won't be created.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_ab_test_variation_with_http_info(ab_test_create_request_v_next, async_req=True)
        >>> result = thread.get()

        :param ab_test_create_request_v_next: (required)
        :type ab_test_create_request_v_next: AbTestCreateRequestVNext
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["ab_test_create_request_v_next"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method create_ab_test_variation" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'ab_test_create_request_v_next' is set
        if self.api_client.client_side_validation and local_var_params.get("ab_test_create_request_v_next") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `ab_test_create_request_v_next` when calling `create_ab_test_variation`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        if "ab_test_create_request_v_next" in local_var_params:
            body_params = local_var_params["ab_test_create_request_v_next"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # HTTP header `Content-Type`
        content_types_list = local_var_params.get("_content_type", self.api_client.select_header_content_type(["application/json"], "POST", body_params))  # noqa: E501
        if content_types_list:
            header_params["Content-Type"] = content_types_list

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            201: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/ab-test/create-variation",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_ab_test_variation(self, email_id, **kwargs):  # noqa: E501
        """Get the variation of a an A/B marketing email  # noqa: E501

        This endpoint lets you obtain the variation of an A/B marketing email. If the email is variation A (master) it will return variation B (variant) and vice versa.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_ab_test_variation(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of an A/B marketing email. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.get_ab_test_variation_with_http_info(email_id, **kwargs)  # noqa: E501

    def get_ab_test_variation_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Get the variation of a an A/B marketing email  # noqa: E501

        This endpoint lets you obtain the variation of an A/B marketing email. If the email is variation A (master) it will return variation B (variant) and vice versa.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_ab_test_variation_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of an A/B marketing email. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_ab_test_variation" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `get_ab_test_variation`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/ab-test/get-variation",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_by_id(self, email_id, **kwargs):  # noqa: E501
        """Get the details of a specified marketing email.  # noqa: E501

        Get the details for a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_by_id(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param include_stats: Include statistics with email
        :type include_stats: bool
        :param marketing_campaign_names:
        :type marketing_campaign_names: bool
        :param workflow_names:
        :type workflow_names: bool
        :param included_properties:
        :type included_properties: list[str]
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.get_by_id_with_http_info(email_id, **kwargs)  # noqa: E501

    def get_by_id_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Get the details of a specified marketing email.  # noqa: E501

        Get the details for a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_by_id_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param include_stats: Include statistics with email
        :type include_stats: bool
        :param marketing_campaign_names:
        :type marketing_campaign_names: bool
        :param workflow_names:
        :type workflow_names: bool
        :param included_properties:
        :type included_properties: list[str]
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "include_stats", "marketing_campaign_names", "workflow_names", "included_properties", "archived"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_by_id" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `get_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []
        if local_var_params.get("include_stats") is not None:  # noqa: E501
            query_params.append(("includeStats", local_var_params["include_stats"]))  # noqa: E501
        if local_var_params.get("marketing_campaign_names") is not None:  # noqa: E501
            query_params.append(("marketingCampaignNames", local_var_params["marketing_campaign_names"]))  # noqa: E501
        if local_var_params.get("workflow_names") is not None:  # noqa: E501
            query_params.append(("workflowNames", local_var_params["workflow_names"]))  # noqa: E501
        if local_var_params.get("included_properties") is not None:  # noqa: E501
            query_params.append(("includedProperties", local_var_params["included_properties"]))  # noqa: E501
            collection_formats["includedProperties"] = "multi"  # noqa: E501
        if local_var_params.get("archived") is not None:  # noqa: E501
            query_params.append(("archived", local_var_params["archived"]))  # noqa: E501

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_draft(self, email_id, **kwargs):  # noqa: E501
        """Get draft version of a marketing email  # noqa: E501

        Get the draft version of an email (if it exists). If no draft version exists, the published email is returned.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_draft(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.get_draft_with_http_info(email_id, **kwargs)  # noqa: E501

    def get_draft_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Get draft version of a marketing email  # noqa: E501

        Get the draft version of an email (if it exists). If no draft version exists, the published email is returned.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_draft_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_draft" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `get_draft`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/draft",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_page(self, **kwargs):  # noqa: E501
        """Get all marketing emails for a HubSpot account.  # noqa: E501

        The results can be filtered, allowing you to find a specific set of emails. See the table below for a full list of filtering options.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_page(async_req=True)
        >>> result = thread.get()

        :param created_at: Only return emails created at exactly the specified time.
        :type created_at: datetime
        :param created_after: Only return emails created after the specified time.
        :type created_after: datetime
        :param created_before: Only return emails created before the specified time.
        :type created_before: datetime
        :param updated_at: Only return emails last updated at exactly the specified time.
        :type updated_at: datetime
        :param updated_after: Only return emails last updated after the specified time.
        :type updated_after: datetime
        :param updated_before: Only return emails last updated before the specified time.
        :type updated_before: datetime
        :param sort: Specifies which fields to use for sorting results. Valid fields are `name`, `createdAt`, `updatedAt`, `createdBy`, `updatedBy`. `createdAt` will be used by default.
        :type sort: list[str]
        :param after: The cursor token value to get the next set of results. You can get this from the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to return. Default is 100.
        :type limit: int
        :param include_stats: Include statistics with emails.
        :type include_stats: bool
        :param marketing_campaign_names:
        :type marketing_campaign_names: bool
        :param workflow_names:
        :type workflow_names: bool
        :param type: Email types to be filtered by. Multiple types can be included. All emails will be returned if not present.
        :type type: str
        :param is_published: Filter by published/draft emails. All emails will be returned if not present.
        :type is_published: bool
        :param included_properties:
        :type included_properties: list[str]
        :param archived: Specifies whether to return archived emails. Defaults to `false`.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CollectionResponseWithTotalPublicEmailForwardPaging
        """
        kwargs["_return_http_data_only"] = True
        return self.get_page_with_http_info(**kwargs)  # noqa: E501

    def get_page_with_http_info(self, **kwargs):  # noqa: E501
        """Get all marketing emails for a HubSpot account.  # noqa: E501

        The results can be filtered, allowing you to find a specific set of emails. See the table below for a full list of filtering options.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_page_with_http_info(async_req=True)
        >>> result = thread.get()

        :param created_at: Only return emails created at exactly the specified time.
        :type created_at: datetime
        :param created_after: Only return emails created after the specified time.
        :type created_after: datetime
        :param created_before: Only return emails created before the specified time.
        :type created_before: datetime
        :param updated_at: Only return emails last updated at exactly the specified time.
        :type updated_at: datetime
        :param updated_after: Only return emails last updated after the specified time.
        :type updated_after: datetime
        :param updated_before: Only return emails last updated before the specified time.
        :type updated_before: datetime
        :param sort: Specifies which fields to use for sorting results. Valid fields are `name`, `createdAt`, `updatedAt`, `createdBy`, `updatedBy`. `createdAt` will be used by default.
        :type sort: list[str]
        :param after: The cursor token value to get the next set of results. You can get this from the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to return. Default is 100.
        :type limit: int
        :param include_stats: Include statistics with emails.
        :type include_stats: bool
        :param marketing_campaign_names:
        :type marketing_campaign_names: bool
        :param workflow_names:
        :type workflow_names: bool
        :param type: Email types to be filtered by. Multiple types can be included. All emails will be returned if not present.
        :type type: str
        :param is_published: Filter by published/draft emails. All emails will be returned if not present.
        :type is_published: bool
        :param included_properties:
        :type included_properties: list[str]
        :param archived: Specifies whether to return archived emails. Defaults to `false`.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CollectionResponseWithTotalPublicEmailForwardPaging, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            "created_at",
            "created_after",
            "created_before",
            "updated_at",
            "updated_after",
            "updated_before",
            "sort",
            "after",
            "limit",
            "include_stats",
            "marketing_campaign_names",
            "workflow_names",
            "type",
            "is_published",
            "included_properties",
            "archived",
        ]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_page" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if local_var_params.get("created_at") is not None:  # noqa: E501
            query_params.append(("createdAt", local_var_params["created_at"]))  # noqa: E501
        if local_var_params.get("created_after") is not None:  # noqa: E501
            query_params.append(("createdAfter", local_var_params["created_after"]))  # noqa: E501
        if local_var_params.get("created_before") is not None:  # noqa: E501
            query_params.append(("createdBefore", local_var_params["created_before"]))  # noqa: E501
        if local_var_params.get("updated_at") is not None:  # noqa: E501
            query_params.append(("updatedAt", local_var_params["updated_at"]))  # noqa: E501
        if local_var_params.get("updated_after") is not None:  # noqa: E501
            query_params.append(("updatedAfter", local_var_params["updated_after"]))  # noqa: E501
        if local_var_params.get("updated_before") is not None:  # noqa: E501
            query_params.append(("updatedBefore", local_var_params["updated_before"]))  # noqa: E501
        if local_var_params.get("sort") is not None:  # noqa: E501
            query_params.append(("sort", local_var_params["sort"]))  # noqa: E501
            collection_formats["sort"] = "multi"  # noqa: E501
        if local_var_params.get("after") is not None:  # noqa: E501
            query_params.append(("after", local_var_params["after"]))  # noqa: E501
        if local_var_params.get("limit") is not None:  # noqa: E501
            query_params.append(("limit", local_var_params["limit"]))  # noqa: E501
        if local_var_params.get("include_stats") is not None:  # noqa: E501
            query_params.append(("includeStats", local_var_params["include_stats"]))  # noqa: E501
        if local_var_params.get("marketing_campaign_names") is not None:  # noqa: E501
            query_params.append(("marketingCampaignNames", local_var_params["marketing_campaign_names"]))  # noqa: E501
        if local_var_params.get("workflow_names") is not None:  # noqa: E501
            query_params.append(("workflowNames", local_var_params["workflow_names"]))  # noqa: E501
        if local_var_params.get("type") is not None:  # noqa: E501
            query_params.append(("type", local_var_params["type"]))  # noqa: E501
        if local_var_params.get("is_published") is not None:  # noqa: E501
            query_params.append(("isPublished", local_var_params["is_published"]))  # noqa: E501
        if local_var_params.get("included_properties") is not None:  # noqa: E501
            query_params.append(("includedProperties", local_var_params["included_properties"]))  # noqa: E501
            collection_formats["includedProperties"] = "multi"  # noqa: E501
        if local_var_params.get("archived") is not None:  # noqa: E501
            query_params.append(("archived", local_var_params["archived"]))  # noqa: E501

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "CollectionResponseWithTotalPublicEmailForwardPaging",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_revision_by_id(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Get a revision of a marketing email.  # noqa: E501

        Get a specific revision of a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_revision_by_id(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: VersionPublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.get_revision_by_id_with_http_info(email_id, revision_id, **kwargs)  # noqa: E501

    def get_revision_by_id_with_http_info(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Get a revision of a marketing email.  # noqa: E501

        Get a specific revision of a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_revision_by_id_with_http_info(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(VersionPublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "revision_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_revision_by_id" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `get_revision_by_id`")  # noqa: E501
        # verify the required parameter 'revision_id' is set
        if self.api_client.client_side_validation and local_var_params.get("revision_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `revision_id` when calling `get_revision_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501
        if "revision_id" in local_var_params:
            path_params["revisionId"] = local_var_params["revision_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "VersionPublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/revisions/{revisionId}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def get_revisions(self, email_id, **kwargs):  # noqa: E501
        """Get revisions of a marketing email  # noqa: E501

        Get a list of all versions of a marketing email, with each entry including the full state of that particular version. The current revision has the ID -1.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_revisions(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param after: The cursor token value to get the next set of results. You can get this from the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param before: The cursor token value to get the previous set of results. You can get this from the `paging.prev.before` JSON property of a paged response containing more results.
        :type before: str
        :param limit: The maximum number of results to return. Default is 100.
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CollectionResponseWithTotalVersionPublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.get_revisions_with_http_info(email_id, **kwargs)  # noqa: E501

    def get_revisions_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Get revisions of a marketing email  # noqa: E501

        Get a list of all versions of a marketing email, with each entry including the full state of that particular version. The current revision has the ID -1.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_revisions_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param after: The cursor token value to get the next set of results. You can get this from the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param before: The cursor token value to get the previous set of results. You can get this from the `paging.prev.before` JSON property of a paged response containing more results.
        :type before: str
        :param limit: The maximum number of results to return. Default is 100.
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CollectionResponseWithTotalVersionPublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "after", "before", "limit"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method get_revisions" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `get_revisions`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []
        if local_var_params.get("after") is not None:  # noqa: E501
            query_params.append(("after", local_var_params["after"]))  # noqa: E501
        if local_var_params.get("before") is not None:  # noqa: E501
            query_params.append(("before", local_var_params["before"]))  # noqa: E501
        if local_var_params.get("limit") is not None:  # noqa: E501
            query_params.append(("limit", local_var_params["limit"]))  # noqa: E501

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "CollectionResponseWithTotalVersionPublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/revisions",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def publish_or_send(self, email_id, **kwargs):  # noqa: E501
        """Publish or send a marketing email.  # noqa: E501

        If you have a Marketing Hub Enterprise account or the transactional email add-on, you can use this endpoint to publish an automated email or send/schedule a regular email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.publish_or_send(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.publish_or_send_with_http_info(email_id, **kwargs)  # noqa: E501

    def publish_or_send_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Publish or send a marketing email.  # noqa: E501

        If you have a Marketing Hub Enterprise account or the transactional email add-on, you can use this endpoint to publish an automated email or send/schedule a regular email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.publish_or_send_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        local_var_params = locals()

        all_params = ["email_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method publish_or_send" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `publish_or_send`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {}

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/publish",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def reset_draft(self, email_id, **kwargs):  # noqa: E501
        """Reset Draft  # noqa: E501

        Resets the draft back to a copy of the live object.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_draft(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.reset_draft_with_http_info(email_id, **kwargs)  # noqa: E501

    def reset_draft_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Reset Draft  # noqa: E501

        Resets the draft back to a copy of the live object.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_draft_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        local_var_params = locals()

        all_params = ["email_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method reset_draft" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `reset_draft`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {}

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/draft/reset",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def restore_draft_revision(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Restore a revision of a marketing email to DRAFT state  # noqa: E501

        Restores a previous revision of a marketing email to DRAFT state. If there is currently something in the draft for that object, it is overwritten.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_draft_revision(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.restore_draft_revision_with_http_info(email_id, revision_id, **kwargs)  # noqa: E501

    def restore_draft_revision_with_http_info(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Restore a revision of a marketing email to DRAFT state  # noqa: E501

        Restores a previous revision of a marketing email to DRAFT state. If there is currently something in the draft for that object, it is overwritten.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_draft_revision_with_http_info(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "revision_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method restore_draft_revision" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `restore_draft_revision`")  # noqa: E501
        # verify the required parameter 'revision_id' is set
        if self.api_client.client_side_validation and local_var_params.get("revision_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `revision_id` when calling `restore_draft_revision`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501
        if "revision_id" in local_var_params:
            path_params["revisionId"] = local_var_params["revision_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/revisions/{revisionId}/restore-to-draft",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def restore_revision(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Restore a revision of a marketing email  # noqa: E501

        Restores a previous revision of a marketing email. The current revision becomes old, and the restored revision is given a new version number.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_revision(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.restore_revision_with_http_info(email_id, revision_id, **kwargs)  # noqa: E501

    def restore_revision_with_http_info(self, email_id, revision_id, **kwargs):  # noqa: E501
        """Restore a revision of a marketing email  # noqa: E501

        Restores a previous revision of a marketing email. The current revision becomes old, and the restored revision is given a new version number.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_revision_with_http_info(email_id, revision_id, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param revision_id: The ID of a revision. (required)
        :type revision_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        local_var_params = locals()

        all_params = ["email_id", "revision_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method restore_revision" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `restore_revision`")  # noqa: E501
        # verify the required parameter 'revision_id' is set
        if self.api_client.client_side_validation and local_var_params.get("revision_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `revision_id` when calling `restore_revision`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501
        if "revision_id" in local_var_params:
            path_params["revisionId"] = local_var_params["revision_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {}

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/revisions/{revisionId}/restore",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def unpublish_or_cancel(self, email_id, **kwargs):  # noqa: E501
        """Unpublish or cancel a marketing email.  # noqa: E501

        If you have a Marketing Hub Enterprise account or the transactional email add-on, you can use this endpoint to unpublish an automated email or cancel a regular email. If the email is already in the process of being sent, canceling might not be possible.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.unpublish_or_cancel(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.unpublish_or_cancel_with_http_info(email_id, **kwargs)  # noqa: E501

    def unpublish_or_cancel_with_http_info(self, email_id, **kwargs):  # noqa: E501
        """Unpublish or cancel a marketing email.  # noqa: E501

        If you have a Marketing Hub Enterprise account or the transactional email add-on, you can use this endpoint to unpublish an automated email or cancel a regular email. If the email is already in the process of being sent, canceling might not be possible.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.unpublish_or_cancel_with_http_info(email_id, async_req=True)
        >>> result = thread.get()

        :param email_id: (required)
        :type email_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        local_var_params = locals()

        all_params = ["email_id"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method unpublish_or_cancel" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `unpublish_or_cancel`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["*/*"])  # noqa: E501

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {}

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/unpublish",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def update(self, email_id, email_update_request, **kwargs):  # noqa: E501
        """Update a marketing email.  # noqa: E501

        Change properties of a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update(email_id, email_update_request, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of the marketing email that should get updated (required)
        :type email_id: str
        :param email_update_request: A marketing email object with properties that should overwrite the corresponding properties of the marketing email. (required)
        :type email_update_request: EmailUpdateRequest
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.update_with_http_info(email_id, email_update_request, **kwargs)  # noqa: E501

    def update_with_http_info(self, email_id, email_update_request, **kwargs):  # noqa: E501
        """Update a marketing email.  # noqa: E501

        Change properties of a marketing email.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_with_http_info(email_id, email_update_request, async_req=True)
        >>> result = thread.get()

        :param email_id: The ID of the marketing email that should get updated (required)
        :type email_id: str
        :param email_update_request: A marketing email object with properties that should overwrite the corresponding properties of the marketing email. (required)
        :type email_update_request: EmailUpdateRequest
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "email_update_request", "archived"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method update" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `update`")  # noqa: E501
        # verify the required parameter 'email_update_request' is set
        if self.api_client.client_side_validation and local_var_params.get("email_update_request") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_update_request` when calling `update`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []
        if local_var_params.get("archived") is not None:  # noqa: E501
            query_params.append(("archived", local_var_params["archived"]))  # noqa: E501

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        if "email_update_request" in local_var_params:
            body_params = local_var_params["email_update_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # HTTP header `Content-Type`
        content_types_list = local_var_params.get("_content_type", self.api_client.select_header_content_type(["application/json"], "PATCH", body_params))  # noqa: E501
        if content_types_list:
            header_params["Content-Type"] = content_types_list

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}",
            "PATCH",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )

    def upsert_draft(self, email_id, email_update_request, **kwargs):  # noqa: E501
        """Create or update draft version  # noqa: E501

        Create or update the draft version of a marketing email. If no draft exists, the system creates a draft from the current “live” email then applies the request body to that draft. The draft version only lives on the buffer—the email is not cloned.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upsert_draft(email_id, email_update_request, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param email_update_request: A marketing email object with properties that should overwrite the corresponding properties in the email's current draft. (required)
        :type email_update_request: EmailUpdateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PublicEmail
        """
        kwargs["_return_http_data_only"] = True
        return self.upsert_draft_with_http_info(email_id, email_update_request, **kwargs)  # noqa: E501

    def upsert_draft_with_http_info(self, email_id, email_update_request, **kwargs):  # noqa: E501
        """Create or update draft version  # noqa: E501

        Create or update the draft version of a marketing email. If no draft exists, the system creates a draft from the current “live” email then applies the request body to that draft. The draft version only lives on the buffer—the email is not cloned.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upsert_draft_with_http_info(email_id, email_update_request, async_req=True)
        >>> result = thread.get()

        :param email_id: The marketing email ID. (required)
        :type email_id: str
        :param email_update_request: A marketing email object with properties that should overwrite the corresponding properties in the email's current draft. (required)
        :type email_update_request: EmailUpdateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PublicEmail, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = ["email_id", "email_update_request"]
        all_params.extend(["async_req", "_return_http_data_only", "_preload_content", "_request_timeout", "_request_auth", "_content_type", "_headers"])

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError("Got an unexpected keyword argument '%s'" " to method upsert_draft" % key)
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'email_id' is set
        if self.api_client.client_side_validation and local_var_params.get("email_id") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_id` when calling `upsert_draft`")  # noqa: E501
        # verify the required parameter 'email_update_request' is set
        if self.api_client.client_side_validation and local_var_params.get("email_update_request") is None:  # noqa: E501
            raise ApiValueError("Missing the required parameter `email_update_request` when calling `upsert_draft`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "email_id" in local_var_params:
            path_params["emailId"] = local_var_params["email_id"]  # noqa: E501

        query_params = []

        header_params = dict(local_var_params.get("_headers", {}))

        form_params = []
        local_var_files = {}

        body_params = None
        if "email_update_request" in local_var_params:
            body_params = local_var_params["email_update_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json", "*/*"])  # noqa: E501

        # HTTP header `Content-Type`
        content_types_list = local_var_params.get("_content_type", self.api_client.select_header_content_type(["application/json"], "PATCH", body_params))  # noqa: E501
        if content_types_list:
            header_params["Content-Type"] = content_types_list

        # Authentication setting
        auth_settings = ["oauth2"]  # noqa: E501

        response_types_map = {
            200: "PublicEmail",
        }

        return self.api_client.call_api(
            "/marketing/v3/emails/{emailId}/draft",
            "PATCH",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get("_request_auth"),
        )



# coding: utf-8

# flake8: noqa
"""
    Postal Mail

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from hubspot.crm.objects.postal_mail.models.associated_id import AssociatedId
from hubspot.crm.objects.postal_mail.models.association_spec import AssociationSpec
from hubspot.crm.objects.postal_mail.models.batch_input_simple_public_object_batch_input import BatchInputSimplePublicObjectBatchInput
from hubspot.crm.objects.postal_mail.models.batch_input_simple_public_object_batch_input_for_create import BatchInputSimplePublicObjectBatchInputForCreate
from hubspot.crm.objects.postal_mail.models.batch_input_simple_public_object_batch_input_upsert import BatchInputSimplePublicObjectBatchInputUpsert
from hubspot.crm.objects.postal_mail.models.batch_input_simple_public_object_id import BatchInputSimplePublicObjectId
from hubspot.crm.objects.postal_mail.models.batch_read_input_simple_public_object_id import BatchReadInputSimplePublicObjectId
from hubspot.crm.objects.postal_mail.models.batch_response_simple_public_object import BatchResponseSimplePublicObject
from hubspot.crm.objects.postal_mail.models.batch_response_simple_public_object_with_errors import BatchResponseSimplePublicObjectWithErrors
from hubspot.crm.objects.postal_mail.models.batch_response_simple_public_upsert_object import BatchResponseSimplePublicUpsertObject
from hubspot.crm.objects.postal_mail.models.batch_response_simple_public_upsert_object_with_errors import BatchResponseSimplePublicUpsertObjectWithErrors
from hubspot.crm.objects.postal_mail.models.collection_response_associated_id import CollectionResponseAssociatedId
from hubspot.crm.objects.postal_mail.models.collection_response_simple_public_object_with_associations_forward_paging import CollectionResponseSimplePublicObjectWithAssociationsForwardPaging
from hubspot.crm.objects.postal_mail.models.collection_response_with_total_simple_public_object_forward_paging import CollectionResponseWithTotalSimplePublicObjectForwardPaging
from hubspot.crm.objects.postal_mail.models.error import Error
from hubspot.crm.objects.postal_mail.models.error_detail import ErrorDetail
from hubspot.crm.objects.postal_mail.models.filter import Filter
from hubspot.crm.objects.postal_mail.models.filter_group import FilterGroup
from hubspot.crm.objects.postal_mail.models.forward_paging import ForwardPaging
from hubspot.crm.objects.postal_mail.models.next_page import NextPage
from hubspot.crm.objects.postal_mail.models.paging import Paging
from hubspot.crm.objects.postal_mail.models.previous_page import PreviousPage
from hubspot.crm.objects.postal_mail.models.public_associations_for_object import PublicAssociationsForObject
from hubspot.crm.objects.postal_mail.models.public_object_id import PublicObjectId
from hubspot.crm.objects.postal_mail.models.public_object_search_request import PublicObjectSearchRequest
from hubspot.crm.objects.postal_mail.models.simple_public_object import SimplePublicObject
from hubspot.crm.objects.postal_mail.models.simple_public_object_batch_input import SimplePublicObjectBatchInput
from hubspot.crm.objects.postal_mail.models.simple_public_object_batch_input_for_create import SimplePublicObjectBatchInputForCreate
from hubspot.crm.objects.postal_mail.models.simple_public_object_batch_input_upsert import SimplePublicObjectBatchInputUpsert
from hubspot.crm.objects.postal_mail.models.simple_public_object_id import SimplePublicObjectId
from hubspot.crm.objects.postal_mail.models.simple_public_object_input import SimplePublicObjectInput
from hubspot.crm.objects.postal_mail.models.simple_public_object_input_for_create import SimplePublicObjectInputForCreate
from hubspot.crm.objects.postal_mail.models.simple_public_object_with_associations import SimplePublicObjectWithAssociations
from hubspot.crm.objects.postal_mail.models.simple_public_upsert_object import SimplePublicUpsertObject
from hubspot.crm.objects.postal_mail.models.standard_error import StandardError
from hubspot.crm.objects.postal_mail.models.value_with_timestamp import ValueWithTimestamp



# coding: utf-8

"""
    Files

    Upload and manage files.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from hubspot.files.configuration import Configuration


class SignedUrl(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"extension": "str", "size": "int", "name": "str", "width": "int", "type": "str", "url": "str", "expires_at": "datetime", "height": "int"}

    attribute_map = {"extension": "extension", "size": "size", "name": "name", "width": "width", "type": "type", "url": "url", "expires_at": "expiresAt", "height": "height"}

    def __init__(self, extension=None, size=None, name=None, width=None, type=None, url=None, expires_at=None, height=None, local_vars_configuration=None):  # noqa: E501
        """SignedUrl - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._extension = None
        self._size = None
        self._name = None
        self._width = None
        self._type = None
        self._url = None
        self._expires_at = None
        self._height = None
        self.discriminator = None

        self.extension = extension
        self.size = size
        self.name = name
        if width is not None:
            self.width = width
        self.type = type
        self.url = url
        self.expires_at = expires_at
        if height is not None:
            self.height = height

    @property
    def extension(self):
        """Gets the extension of this SignedUrl.  # noqa: E501

        Extension of the requested file.  # noqa: E501

        :return: The extension of this SignedUrl.  # noqa: E501
        :rtype: str
        """
        return self._extension

    @extension.setter
    def extension(self, extension):
        """Sets the extension of this SignedUrl.

        Extension of the requested file.  # noqa: E501

        :param extension: The extension of this SignedUrl.  # noqa: E501
        :type extension: str
        """
        if self.local_vars_configuration.client_side_validation and extension is None:  # noqa: E501
            raise ValueError("Invalid value for `extension`, must not be `None`")  # noqa: E501

        self._extension = extension

    @property
    def size(self):
        """Gets the size of this SignedUrl.  # noqa: E501

        Size in bytes of the requested file.  # noqa: E501

        :return: The size of this SignedUrl.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this SignedUrl.

        Size in bytes of the requested file.  # noqa: E501

        :param size: The size of this SignedUrl.  # noqa: E501
        :type size: int
        """
        if self.local_vars_configuration.client_side_validation and size is None:  # noqa: E501
            raise ValueError("Invalid value for `size`, must not be `None`")  # noqa: E501

        self._size = size

    @property
    def name(self):
        """Gets the name of this SignedUrl.  # noqa: E501

        Name of the requested file.  # noqa: E501

        :return: The name of this SignedUrl.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SignedUrl.

        Name of the requested file.  # noqa: E501

        :param name: The name of this SignedUrl.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def width(self):
        """Gets the width of this SignedUrl.  # noqa: E501

        For image and video files. The width of the file.  # noqa: E501

        :return: The width of this SignedUrl.  # noqa: E501
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """Sets the width of this SignedUrl.

        For image and video files. The width of the file.  # noqa: E501

        :param width: The width of this SignedUrl.  # noqa: E501
        :type width: int
        """

        self._width = width

    @property
    def type(self):
        """Gets the type of this SignedUrl.  # noqa: E501

        Type of the file. Can be IMG, DOCUMENT, AUDIO, MOVIE, or OTHER.  # noqa: E501

        :return: The type of this SignedUrl.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this SignedUrl.

        Type of the file. Can be IMG, DOCUMENT, AUDIO, MOVIE, or OTHER.  # noqa: E501

        :param type: The type of this SignedUrl.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def url(self):
        """Gets the url of this SignedUrl.  # noqa: E501

        Signed URL with access to the specified file. Anyone with this URL will be able to access the file until it expires.  # noqa: E501

        :return: The url of this SignedUrl.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this SignedUrl.

        Signed URL with access to the specified file. Anyone with this URL will be able to access the file until it expires.  # noqa: E501

        :param url: The url of this SignedUrl.  # noqa: E501
        :type url: str
        """
        if self.local_vars_configuration.client_side_validation and url is None:  # noqa: E501
            raise ValueError("Invalid value for `url`, must not be `None`")  # noqa: E501

        self._url = url

    @property
    def expires_at(self):
        """Gets the expires_at of this SignedUrl.  # noqa: E501

        Timestamp of when the URL will no longer grant access to the file.  # noqa: E501

        :return: The expires_at of this SignedUrl.  # noqa: E501
        :rtype: datetime
        """
        return self._expires_at

    @expires_at.setter
    def expires_at(self, expires_at):
        """Sets the expires_at of this SignedUrl.

        Timestamp of when the URL will no longer grant access to the file.  # noqa: E501

        :param expires_at: The expires_at of this SignedUrl.  # noqa: E501
        :type expires_at: datetime
        """
        if self.local_vars_configuration.client_side_validation and expires_at is None:  # noqa: E501
            raise ValueError("Invalid value for `expires_at`, must not be `None`")  # noqa: E501

        self._expires_at = expires_at

    @property
    def height(self):
        """Gets the height of this SignedUrl.  # noqa: E501

        For image and video files. The height of the file.  # noqa: E501

        :return: The height of this SignedUrl.  # noqa: E501
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """Sets the height of this SignedUrl.

        For image and video files. The height of the file.  # noqa: E501

        :param height: The height of this SignedUrl.  # noqa: E501
        :type height: int
        """

        self._height = height

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(lambda x: convert(x), value))
            elif isinstance(value, dict):
                result[attr] = dict(map(lambda item: (item[0], convert(item[1])), value.items()))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SignedUrl):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SignedUrl):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Video Conferencing Extension

    These APIs allow you to specify URLs that can be used to interact with a video conferencing application, to allow HubSpot to add video conference links to meeting requests with contacts.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from hubspot.crm.extensions.videoconferencing.configuration import Configuration


class ExternalSettings(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"user_verify_url": "str", "fetch_accounts_uri": "str", "create_meeting_url": "str", "update_meeting_url": "str", "delete_meeting_url": "str"}

    attribute_map = {
        "user_verify_url": "userVerifyUrl",
        "fetch_accounts_uri": "fetchAccountsUri",
        "create_meeting_url": "createMeetingUrl",
        "update_meeting_url": "updateMeetingUrl",
        "delete_meeting_url": "deleteMeetingUrl",
    }

    def __init__(self, user_verify_url=None, fetch_accounts_uri=None, create_meeting_url=None, update_meeting_url=None, delete_meeting_url=None, local_vars_configuration=None):  # noqa: E501
        """ExternalSettings - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._user_verify_url = None
        self._fetch_accounts_uri = None
        self._create_meeting_url = None
        self._update_meeting_url = None
        self._delete_meeting_url = None
        self.discriminator = None

        if user_verify_url is not None:
            self.user_verify_url = user_verify_url
        if fetch_accounts_uri is not None:
            self.fetch_accounts_uri = fetch_accounts_uri
        self.create_meeting_url = create_meeting_url
        if update_meeting_url is not None:
            self.update_meeting_url = update_meeting_url
        if delete_meeting_url is not None:
            self.delete_meeting_url = delete_meeting_url

    @property
    def user_verify_url(self):
        """Gets the user_verify_url of this ExternalSettings.  # noqa: E501

        The URL that HubSpot will use to verify that a user exists in the video conference application.  # noqa: E501

        :return: The user_verify_url of this ExternalSettings.  # noqa: E501
        :rtype: str
        """
        return self._user_verify_url

    @user_verify_url.setter
    def user_verify_url(self, user_verify_url):
        """Sets the user_verify_url of this ExternalSettings.

        The URL that HubSpot will use to verify that a user exists in the video conference application.  # noqa: E501

        :param user_verify_url: The user_verify_url of this ExternalSettings.  # noqa: E501
        :type user_verify_url: str
        """

        self._user_verify_url = user_verify_url

    @property
    def fetch_accounts_uri(self):
        """Gets the fetch_accounts_uri of this ExternalSettings.  # noqa: E501


        :return: The fetch_accounts_uri of this ExternalSettings.  # noqa: E501
        :rtype: str
        """
        return self._fetch_accounts_uri

    @fetch_accounts_uri.setter
    def fetch_accounts_uri(self, fetch_accounts_uri):
        """Sets the fetch_accounts_uri of this ExternalSettings.


        :param fetch_accounts_uri: The fetch_accounts_uri of this ExternalSettings.  # noqa: E501
        :type fetch_accounts_uri: str
        """

        self._fetch_accounts_uri = fetch_accounts_uri

    @property
    def create_meeting_url(self):
        """Gets the create_meeting_url of this ExternalSettings.  # noqa: E501

        The URL that HubSpot will send requests to create a new video conference.  # noqa: E501

        :return: The create_meeting_url of this ExternalSettings.  # noqa: E501
        :rtype: str
        """
        return self._create_meeting_url

    @create_meeting_url.setter
    def create_meeting_url(self, create_meeting_url):
        """Sets the create_meeting_url of this ExternalSettings.

        The URL that HubSpot will send requests to create a new video conference.  # noqa: E501

        :param create_meeting_url: The create_meeting_url of this ExternalSettings.  # noqa: E501
        :type create_meeting_url: str
        """
        if self.local_vars_configuration.client_side_validation and create_meeting_url is None:  # noqa: E501
            raise ValueError("Invalid value for `create_meeting_url`, must not be `None`")  # noqa: E501

        self._create_meeting_url = create_meeting_url

    @property
    def update_meeting_url(self):
        """Gets the update_meeting_url of this ExternalSettings.  # noqa: E501

        The URL that HubSpot will send updates to existing meetings. Typically called when the user changes the topic or times of a meeting.  # noqa: E501

        :return: The update_meeting_url of this ExternalSettings.  # noqa: E501
        :rtype: str
        """
        return self._update_meeting_url

    @update_meeting_url.setter
    def update_meeting_url(self, update_meeting_url):
        """Sets the update_meeting_url of this ExternalSettings.

        The URL that HubSpot will send updates to existing meetings. Typically called when the user changes the topic or times of a meeting.  # noqa: E501

        :param update_meeting_url: The update_meeting_url of this ExternalSettings.  # noqa: E501
        :type update_meeting_url: str
        """

        self._update_meeting_url = update_meeting_url

    @property
    def delete_meeting_url(self):
        """Gets the delete_meeting_url of this ExternalSettings.  # noqa: E501

        The URL that HubSpot will send notifications of meetings that have been deleted in HubSpot.  # noqa: E501

        :return: The delete_meeting_url of this ExternalSettings.  # noqa: E501
        :rtype: str
        """
        return self._delete_meeting_url

    @delete_meeting_url.setter
    def delete_meeting_url(self, delete_meeting_url):
        """Sets the delete_meeting_url of this ExternalSettings.

        The URL that HubSpot will send notifications of meetings that have been deleted in HubSpot.  # noqa: E501

        :param delete_meeting_url: The delete_meeting_url of this ExternalSettings.  # noqa: E501
        :type delete_meeting_url: str
        """

        self._delete_meeting_url = delete_meeting_url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(lambda x: convert(x), value))
            elif isinstance(value, dict):
                result[attr] = dict(map(lambda item: (item[0], convert(item[1])), value.items()))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ExternalSettings):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ExternalSettings):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa
"""
    Business Units

    Retrieve Business Unit information.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from hubspot.settings.business_units.models.collection_response_public_business_unit_no_paging import CollectionResponsePublicBusinessUnitNoPaging
from hubspot.settings.business_units.models.error import Error
from hubspot.settings.business_units.models.error_detail import ErrorDetail
from hubspot.settings.business_units.models.public_business_unit import PublicBusinessUnit
from hubspot.settings.business_units.models.public_business_unit_logo_metadata import PublicBusinessUnitLogoMetadata



import hubspot.crm.associations.v4 as api_client
from ....discovery_base import DiscoveryBase


class Discovery(DiscoveryBase):
    @property
    def basic_api(self) -> api_client.BasicApi:
        return self._configure_api_client(api_client, "BasicApi")

    @property
    def batch_api(self) -> api_client.BatchApi:
        return self._configure_api_client(api_client, "BatchApi")

    @property
    def report_api(self) -> api_client.ReportApi:
        return self._configure_api_client(api_client, "ReportApi")

    @property
    def schema(self):
        from .schema.discovery import Discovery as schemaDiscovery
        return schemaDiscovery(self.config)



# coding: utf-8

# flake8: noqa

"""
    Forms

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from hubspot.marketing.forms.api.forms_api import FormsApi

# import ApiClient
from hubspot.marketing.forms.api_client import ApiClient
from hubspot.marketing.forms.configuration import Configuration
from hubspot.marketing.forms.exceptions import OpenApiException
from hubspot.marketing.forms.exceptions import ApiTypeError
from hubspot.marketing.forms.exceptions import ApiValueError
from hubspot.marketing.forms.exceptions import ApiKeyError
from hubspot.marketing.forms.exceptions import ApiAttributeError
from hubspot.marketing.forms.exceptions import ApiException

# import models into sdk package
from hubspot.marketing.forms.models.collection_response_form_definition_base_forward_paging import CollectionResponseFormDefinitionBaseForwardPaging
from hubspot.marketing.forms.models.collection_response_form_definition_base_forward_paging_results_inner import CollectionResponseFormDefinitionBaseForwardPagingResultsInner
from hubspot.marketing.forms.models.datepicker_field import DatepickerField
from hubspot.marketing.forms.models.dependent_field import DependentField
from hubspot.marketing.forms.models.dependent_field_dependent_field import DependentFieldDependentField
from hubspot.marketing.forms.models.dependent_field_filter import DependentFieldFilter
from hubspot.marketing.forms.models.dropdown_field import DropdownField
from hubspot.marketing.forms.models.email_field import EmailField
from hubspot.marketing.forms.models.email_field_validation import EmailFieldValidation
from hubspot.marketing.forms.models.enumerated_field_option import EnumeratedFieldOption
from hubspot.marketing.forms.models.error import Error
from hubspot.marketing.forms.models.error_detail import ErrorDetail
from hubspot.marketing.forms.models.field_group import FieldGroup
from hubspot.marketing.forms.models.file_field import FileField
from hubspot.marketing.forms.models.form_definition_base import FormDefinitionBase
from hubspot.marketing.forms.models.form_definition_create_request_base import FormDefinitionCreateRequestBase
from hubspot.marketing.forms.models.form_display_options import FormDisplayOptions
from hubspot.marketing.forms.models.form_post_submit_action import FormPostSubmitAction
from hubspot.marketing.forms.models.form_style import FormStyle
from hubspot.marketing.forms.models.forward_paging import ForwardPaging
from hubspot.marketing.forms.models.hub_spot_form_configuration import HubSpotFormConfiguration
from hubspot.marketing.forms.models.hub_spot_form_definition import HubSpotFormDefinition
from hubspot.marketing.forms.models.hub_spot_form_definition_all_of import HubSpotFormDefinitionAllOf
from hubspot.marketing.forms.models.hub_spot_form_definition_all_of_legal_consent_options import HubSpotFormDefinitionAllOfLegalConsentOptions
from hubspot.marketing.forms.models.hub_spot_form_definition_create_request import HubSpotFormDefinitionCreateRequest
from hubspot.marketing.forms.models.hub_spot_form_definition_create_request_all_of import HubSpotFormDefinitionCreateRequestAllOf
from hubspot.marketing.forms.models.hub_spot_form_definition_patch_request import HubSpotFormDefinitionPatchRequest
from hubspot.marketing.forms.models.legal_consent_checkbox import LegalConsentCheckbox
from hubspot.marketing.forms.models.legal_consent_options_explicit_consent_to_process import LegalConsentOptionsExplicitConsentToProcess
from hubspot.marketing.forms.models.legal_consent_options_implicit_consent_to_process import LegalConsentOptionsImplicitConsentToProcess
from hubspot.marketing.forms.models.legal_consent_options_legitimate_interest import LegalConsentOptionsLegitimateInterest
from hubspot.marketing.forms.models.legal_consent_options_none import LegalConsentOptionsNone
from hubspot.marketing.forms.models.lifecycle_stage import LifecycleStage
from hubspot.marketing.forms.models.mobile_phone_field import MobilePhoneField
from hubspot.marketing.forms.models.multi_line_text_field import MultiLineTextField
from hubspot.marketing.forms.models.multiple_checkboxes_field import MultipleCheckboxesField
from hubspot.marketing.forms.models.next_page import NextPage
from hubspot.marketing.forms.models.number_field import NumberField
from hubspot.marketing.forms.models.number_field_validation import NumberFieldValidation
from hubspot.marketing.forms.models.payment_link_radio_field import PaymentLinkRadioField
from hubspot.marketing.forms.models.phone_field import PhoneField
from hubspot.marketing.forms.models.phone_field_validation import PhoneFieldValidation
from hubspot.marketing.forms.models.radio_field import RadioField
from hubspot.marketing.forms.models.single_checkbox_field import SingleCheckboxField
from hubspot.marketing.forms.models.single_line_text_field import SingleLineTextField



# coding: utf-8

# flake8: noqa

"""
    Schemas

    The CRM uses schemas to define how custom objects should store and represent information in the HubSpot CRM. Schemas define details about an object's type, properties, and associations. The schema can be uniquely identified by its **object type ID**.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from hubspot.crm.schemas.api.core_api import CoreApi

# import ApiClient
from hubspot.crm.schemas.api_client import ApiClient
from hubspot.crm.schemas.configuration import Configuration
from hubspot.crm.schemas.exceptions import OpenApiException
from hubspot.crm.schemas.exceptions import ApiTypeError
from hubspot.crm.schemas.exceptions import ApiValueError
from hubspot.crm.schemas.exceptions import ApiKeyError
from hubspot.crm.schemas.exceptions import ApiAttributeError
from hubspot.crm.schemas.exceptions import ApiException

# import models into sdk package
from hubspot.crm.schemas.models.association_definition import AssociationDefinition
from hubspot.crm.schemas.models.association_definition_egg import AssociationDefinitionEgg
from hubspot.crm.schemas.models.collection_response_object_schema_no_paging import CollectionResponseObjectSchemaNoPaging
from hubspot.crm.schemas.models.error import Error
from hubspot.crm.schemas.models.error_detail import ErrorDetail
from hubspot.crm.schemas.models.model_property import ModelProperty
from hubspot.crm.schemas.models.object_schema import ObjectSchema
from hubspot.crm.schemas.models.object_schema_egg import ObjectSchemaEgg
from hubspot.crm.schemas.models.object_type_definition import ObjectTypeDefinition
from hubspot.crm.schemas.models.object_type_definition_labels import ObjectTypeDefinitionLabels
from hubspot.crm.schemas.models.object_type_definition_patch import ObjectTypeDefinitionPatch
from hubspot.crm.schemas.models.object_type_property_create import ObjectTypePropertyCreate
from hubspot.crm.schemas.models.option import Option
from hubspot.crm.schemas.models.option_input import OptionInput
from hubspot.crm.schemas.models.property_modification_metadata import PropertyModificationMetadata



# coding: utf-8

# flake8: noqa

"""
    Properties

    All HubSpot objects store data in default and custom properties. These endpoints provide access to read and modify object properties in HubSpot.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from hubspot.crm.properties.api.batch_api import BatchApi
from hubspot.crm.properties.api.core_api import CoreApi
from hubspot.crm.properties.api.groups_api import GroupsApi

# import ApiClient
from hubspot.crm.properties.api_client import ApiClient
from hubspot.crm.properties.configuration import Configuration
from hubspot.crm.properties.exceptions import OpenApiException
from hubspot.crm.properties.exceptions import ApiTypeError
from hubspot.crm.properties.exceptions import ApiValueError
from hubspot.crm.properties.exceptions import ApiKeyError
from hubspot.crm.properties.exceptions import ApiAttributeError
from hubspot.crm.properties.exceptions import ApiException

# import models into sdk package
from hubspot.crm.properties.models.batch_input_property_create import BatchInputPropertyCreate
from hubspot.crm.properties.models.batch_input_property_name import BatchInputPropertyName
from hubspot.crm.properties.models.batch_read_input_property_name import BatchReadInputPropertyName
from hubspot.crm.properties.models.batch_response_property import BatchResponseProperty
from hubspot.crm.properties.models.batch_response_property_with_errors import BatchResponsePropertyWithErrors
from hubspot.crm.properties.models.collection_response_property_group_no_paging import CollectionResponsePropertyGroupNoPaging
from hubspot.crm.properties.models.collection_response_property_no_paging import CollectionResponsePropertyNoPaging
from hubspot.crm.properties.models.error import Error
from hubspot.crm.properties.models.error_detail import ErrorDetail
from hubspot.crm.properties.models.model_property import ModelProperty
from hubspot.crm.properties.models.option import Option
from hubspot.crm.properties.models.option_input import OptionInput
from hubspot.crm.properties.models.property_create import PropertyCreate
from hubspot.crm.properties.models.property_group import PropertyGroup
from hubspot.crm.properties.models.property_group_create import PropertyGroupCreate
from hubspot.crm.properties.models.property_group_update import PropertyGroupUpdate
from hubspot.crm.properties.models.property_modification_metadata import PropertyModificationMetadata
from hubspot.crm.properties.models.property_name import PropertyName
from hubspot.crm.properties.models.property_update import PropertyUpdate
from hubspot.crm.properties.models.standard_error import StandardError



import hubspot.cms.blogs.tags as api_client
from ....discovery_base import DiscoveryBase


class Discovery(DiscoveryBase):
    @property
    def tag_api(self) -> api_client.BlogTagsApi:
        return self._configure_api_client(api_client, "BlogTagsApi")
